//
//  AppCategoryViewController.swift
//  iAppsReview
//
//  Created by Kevin McNeish on 10/3/14.
//  Copyright (c) 2014 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit
import CoreData

protocol AppCategoryDelegate {
    
    func updateAppCategory(appCategoryEntity: AppCategoryEntity)
}


class AppCategoryViewController: UITableViewController {
    
    var appCategoryID: NSNumber?
    var delegate: AppCategoryDelegate?
    var appCategoryList = Array<AppCategoryEntity>()
    var appCategory = AppCategory()
    
    var oldIndexPath: NSIndexPath?

    override func viewDidLoad() {
        super.viewDidLoad()
        self.appCategoryList = appCategory.getAllEntities()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        // Return the number of sections.
        return 1
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // Return the number of rows in the section.
        return self.appCategoryList.count
    }

    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("CategoryCell", forIndexPath: indexPath) as UITableViewCell

        // Configure the cell...
        let categoryEntity: AppCategoryEntity =
            self.appCategoryList[indexPath.row] as AppCategoryEntity
        
        cell.textLabel!.text = categoryEntity.name
        
        // Check/uncheck the current cell
        if categoryEntity.categoryID.integerValue == self.appCategoryID?.integerValue
        {
            cell.accessoryType = UITableViewCellAccessoryType.Checkmark
            oldIndexPath = indexPath
        }
        else
        {
            cell.accessoryType = UITableViewCellAccessoryType.None
        }
        return cell
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        // Deselect the currently selected row
        self.tableView.deselectRowAtIndexPath(self.tableView.indexPathForSelectedRow!, animated: false)
        
        // Uncheck the previously selected cell, check the currently selected cell
        if  let i = oldIndexPath
        {
            tableView.cellForRowAtIndexPath(i)?.accessoryType =
                UITableViewCellAccessoryType.None
        }
        
        tableView.cellForRowAtIndexPath(indexPath)?.accessoryType =
            UITableViewCellAccessoryType.Checkmark
        
        // Save the current indexPath for later
        oldIndexPath = indexPath
        
        // Send a callback message to the delgate (presenting view controller)
        let categoryEntity: AppCategoryEntity = self.appCategoryList[indexPath.row]
            as AppCategoryEntity
        
        self.delegate?.updateAppCategory(categoryEntity)
    }

}
